﻿using System.Runtime.InteropServices;
using Microsoft.Office.Interop.Word;
using System;

namespace Converter
{
    /// <summary>
    /// This is the converter class for Office and Word 2007 only!
    /// 
    /// You have to install the plug-in to save in PDF and XPS before
    /// 
    /// If you have a trial edition of Word 2007 you cannot use this converter beacuse 
    /// Microsoft doesn't allow you to invoke the COM in a trial edition. Sorry!
    /// </summary>
    class WordConverter:AbstractConverter
    {
        private Microsoft.Office.Interop.Word.ApplicationClass WordApp;
        private bool visible;

        /// <summary>
        /// Close the converter
        /// </summary>
        public override void CloseConverter()
        {
            object missing = System.Reflection.Missing.Value;
            object saveChanges = false;
            WordApp.Quit(ref saveChanges, ref missing, ref missing);
        }

        /// <summary>
        /// Open the converter
        /// </summary>
        /// <param name="visible"></param>
        public override void OpenConverter(bool visible)
        {
            this.visible = visible;
            WordApp = new Microsoft.Office.Interop.Word.ApplicationClass();
            WordApp.Visible = this.visible;
        }

        /// <summary>
        /// Conversion in Word 2007
        /// </summary>
        /// <param name="fileInput"></param>
        /// <param name="fileOutput"></param>
        /// <exception cref="COMException"></exception>
        /// <returns></returns>
        public override string ConvDoc(string fileInput, string fileOutput)
        {
            string ret = null;
            Microsoft.Office.Interop.Word.Document aDoc = null;

            // you can add more settings if you want
            object fileName = fileInput;
            object readOnly = true;
            object isVisible = this.visible;
            object missing = System.Reflection.Missing.Value;
            object format = WdOpenFormat.wdOpenFormatWebPages;

            try
            {
                aDoc = WordApp.Documents.Open(ref fileName,ref missing,ref readOnly,ref missing,
                                            ref missing,ref missing,ref missing,ref missing,ref missing,
                                            ref format,ref missing,ref isVisible,ref missing,ref missing,
                                            ref missing,ref missing);
            }
            catch (COMException e)
            {
                ret = "Word2007Converter: problems reading the input file";
                Console.WriteLine(e.StackTrace);
                return ret;
            }

            object s = fileOutput;
            object addToRecent = false;
            object saveFormat = WdSaveFormat.wdFormatPDF;
            try
            {
                aDoc.SaveAs(ref s,ref saveFormat,ref missing,ref missing,ref addToRecent,ref missing,
                            ref missing,ref missing,ref missing,ref missing,ref missing,ref missing,
                            ref missing,ref missing,ref missing,ref missing);
            }
            catch (COMException e)
            {
                ret = "Word2007Converter: problems writing the output file";
                Console.WriteLine(e.StackTrace);
            }

            object saveChanges = false;
            ((_Document) aDoc).Close(ref saveChanges, ref missing, ref missing);

            return ret;
        }
    }
}
