﻿using unoidl.com.sun.star.lang;
using unoidl.com.sun.star.frame;
using unoidl.com.sun.star.beans;
using uno;
using System.Runtime.InteropServices;
using unoidl.com.sun.star.task;

namespace Converter
{

        /// <summary>
        /// Converter for Writer in OpenOffice 2.4.2
        /// 
        /// OpenOffice has a lot of problems in the conversion process. Sometime
        /// the soffice.bin hangs. it is very strange!
        /// 
        /// Don't trust OpenOffice for conversion in PDF. It isn't much reliable!
        /// </summary>
        internal class WriterConverter:AbstractConverter
        {
            private unoidl.com.sun.star.uno.XComponentContext localContext = null;
            private XComponentLoader componentLoader;
            private bool visible;

            /// <summary>
            /// Close the conversion
            /// 
            /// it doesn't nothing
            /// </summary>
            public override void CloseConverter()
            {

            }

            /// <summary>
            /// Open the converter
            /// </summary>
            /// <exception cref="SEHException"></exception>
            public override void OpenConverter(bool visible)
            {
                this.visible = visible;

                //Call the bootstrap method to get a new ComponentContext
                //object. If OpenOffice isn't already started this will
                //start it and then return the ComponentContext.
                try
                {
                    localContext = uno.util.Bootstrap.bootstrap();
                }
                catch (SEHException)
                {
                    // Only thrown in OpenOffice 3.0 if you don't set the environment variable
                    throw;
                }

                //Get a new service manager of the MultiServiceFactory type
                //we need this to get a desktop object and create new CLI
                //objects.
                unoidl.com.sun.star.lang.XMultiServiceFactory multiServiceFactory = (unoidl.com.sun.star.lang.XMultiServiceFactory)
                localContext.getServiceManager();

                //Create a new Desktop instance using our service manager
                //Notice: We cast our desktop object to XComponent loader
                //so that we could load or create new documents.
                componentLoader = (XComponentLoader)multiServiceFactory.createInstance("com.sun.star.frame.Desktop");
            }


            /// <summary>
            /// Convert the document
            /// </summary>
            /// <param name="fileInput"></param>
            /// <param name="fileOutput"></param>
            /// <exception cref="SEHException"></exception>
            public override string ConvDoc(string fileInput, string fileOutput)
            {
                PropertyValue[] propertyValue = new PropertyValue[1];
                PropertyValue aProperty = new PropertyValue();
                if (visible == false)
                {
                    aProperty.Name = "Hidden";
                    aProperty.Value = new uno.Any(true);
                }
                propertyValue[0] = aProperty;

                XComponent xComponent = null;
                try
                {
                        xComponent = componentLoader.loadComponentFromURL(PathConverter(fileInput), //a blank writer document
                                                                          "_blank", 0, //into a blank frame use no searchflag
                                                                          //use no additional arguments.                                                                        
                                                                          propertyValue);
                }
                catch (ErrorCodeIOException)
                {
                    return "OpenOfficeConverter: problems reading the input file";
                }

                PropertyValue[] saveProps = new PropertyValue[1];
                saveProps[0] = new PropertyValue();
                saveProps[0].Name = "FilterName";
                saveProps[0].Value = new Any("writer_pdf_Export");

                try
                {
                    ((XStorable)xComponent).storeToURL(PathConverter(fileOutput),saveProps);
                }
                catch (ErrorCodeIOException)
                {
                    return "OpenOfficeConverter: problems writing the output file";
                }
                xComponent.dispose();
                return null;
            }


            /// <summary>
            /// Convert into OO file format
            /// </summary>
            /// <param name="file">The file.</param>
            /// <returns>The converted file</returns>
            private static string PathConverter(string file)
            {
                try
                {
                    file = file.Replace(@"\", "/");
                    return "file:///" + file;
                }
                catch (System.Exception)
                {
                    throw;
                }
            }


        }

}
