﻿using System;
using System.Windows.Forms;
using Converter;

namespace WindowsFormsHTM2PDF
{
    /// <summary>
    /// MainForm 
    /// </summary>
    public partial class MainForm : Form
    {
        /// <summary>
        /// Principal and unique Form
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            RunConversion();
        }

        /// <summary>
        /// Core method that invokes all the conversion logic
        /// </summary>
        private void RunConversion()
        {
            string ret = null;
            IConverter pdo = null;
            int soglia = Convert.ToInt32(this.textBoxNumAfterKill.Text);
            bool visible = checkBoxVisibleApp.Checked;
            int upperboundError = Convert.ToInt32(this.textBoxNumErrors.Text);

            if (checkBoxWord.Checked == true || checkBoxWriter.Checked == true)
            {
                string[] processNames;
                this.buttonGo.Enabled = false;                
                listBoxResult.Items.Clear();

                // Set the parameters before the conversion starts
                if (checkBoxWord.Checked == true)
                {
                    pdo = new WordConverter();
                    processNames = new string[1];
                    processNames[0] = "winword";

                }
                else if (checkBoxWriter.Checked == true)
                {
                    pdo = new WriterConverter();
                    processNames = new string[1];
                    processNames[0] = "soffice.bin";
                }
                else
                {
                    listBoxResult.Items.Add("Choose the using converter");
                    this.buttonGo.Enabled = true;
                    return;
                }

                // Open  the converter

                try
                {
                    pdo.OpenConverter(visible);
                }
                catch (Exception)
                {
                    MessageBox.Show("Have you installed OpenOffice or Word?");
                    Application.Exit();
                }

                if (this.openFileDialog1.ShowDialog() == DialogResult.OK)
                {
                    int j = 1;
                    int countException = 0;

                    if (checkBoxStressTest.Checked == true)
                        j = Convert.ToInt32(textBoxNumStressTest.Text);

                    for (int i = 0; i < j; i++)
                    {
                        string tmpPath = openFileDialog1.FileName + ".pdf";

                        // After some steps if you want you can kill the process
                        if (soglia > 0 && i % soglia == 0)
                        {
                            CloseKillOpen(pdo, processNames, visible);
                        }

                        // Start the conversion
                        int countError = 0;
                        do
                        {
                            ret = pdo.ConvDoc(openFileDialog1.FileName, tmpPath);
                            if (ret != null)
                            {
                                CloseKillOpen(pdo, processNames,  visible);
                                listBoxResult.Items.Add(ret + " " + (i + 1));                                
                                countError++;
                            }
                            else
                            {
                                countError=0;
                                listBoxResult.Items.Add(String.Format("Opened and converted successfully [{0}]: " + (i + 1), tmpPath));
                            }

                        }
                        while (ret != null && countError <= upperboundError);

                        if (countError > 0) countException++;

                        listBoxResult.SelectedIndex = listBoxResult.Items.Count - 1;
                        System.Windows.Forms.Application.DoEvents();

                    }

                    if (countException == 0)
                        listBoxResult.Items.Add("Conversion terminated successfully");
                    else
                        listBoxResult.Items.Add("Conversion terminated WITH PROBLEM: " + countException);

                    listBoxResult.SelectedIndex = listBoxResult.Items.Count - 1;
                }

                // Close the converter
                pdo.CloseConverter();

                this.buttonGo.Enabled = true;
            }
            else
            {
                listBoxResult.Items.Add("You have to select the converter");
            }
        }

        private void CloseKillOpen(IConverter pdo, string[] processNames, bool visible)
        {
            pdo.CloseConverter();
            pdo.Kill(processNames);
            listBoxResult.Items.Add("Process kill");

            try
            {
                pdo.OpenConverter(visible);
            }
            catch (Exception)
            {
                MessageBox.Show("Have you installed OpenOffice or Word?");
                Application.Exit();
            }
        }

       

        private void checkBoxWriter_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxStressTest.Checked == true)
            {
                this.textBoxNumStressTest.Enabled = true;
                this.textBoxNumAfterKill.Enabled = true;
            }
            else
            {
                this.textBoxNumStressTest.Enabled = false;
                this.textBoxNumAfterKill.Enabled = false;
            }
        }

        private void checkBoxWord_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxStressTest.Checked == true)
            {
                this.textBoxNumStressTest.Enabled = true;
                this.textBoxNumAfterKill.Enabled = true;
            }
            else
            {
                this.textBoxNumStressTest.Enabled = false;
                this.textBoxNumAfterKill.Enabled = false;
            }
        }

    }
}
